# Firebase Firestore Quickstart

The Firebase Firestore Test Application (testapp) demonstrates Firebase
Firestore operations with the Firebase Firestore C++ SDK. The application has no
user interface and simply logs actions it's performing to the console.

The testapp performs the following:

-   Creates a firebase::App in a platform-specific way. The App holds
    platform-specific context that's used by other Firebase APIs, and is a
    central point for communication between the Firebase Firestore C++ and
    Firebase Auth C++ libraries.
-   Gets a pointer to firebase::Auth, and signs in anonymously. This allows the
    testapp to access a Firebase Firestore instance with authentication rules
    enabled.
-   Initializes a Firestore instance and sets its logging level to
    `kLogLevelDebug` in order to see debug messages in the logs.
-   Tests that it can create `Timestamp`, `SnapshotMetadata`, and `GeoPoint`
    objects.
-   Creates a collection and a document inside that collection.
-   Writes initial data to the document (`Set`), updates the document content
    (`Update`), reads the document back (`Get`), and checks that the contents
    match our expectation.
-   Deletes the document.
-   Performs a batch write to two documents.
-   Performs a Transaction containing three operations (`Update`, `Delete`, and
    `Set`) on three different documents.
-   Queries documents in the collection that match a certain condition. Ensures
    the documents returned via the query match our expectation.

Introduction
------------

-   [Read more about Firebase Firestore](https://firebase.google.com/docs/firestore/)

Building and Running the testapp
--------------------------------

### iOS

-   Link your iOS app to the Firebase libraries.

    -   Get CocoaPods version 1 or later by running,

        ```
          sudo gem install cocoapods --pre
        ```

    -   Update the pod versions in the Podfile to match the C++ SDK version that you are using.
        For the latest version of the C++ SDK, you can find the associated pod versions on the
        [`Add Firebase to your project` page](https://firebase.google.com/docs/cpp/setup?platform=ios#libraries-ios).
        For instance: if you're using SDK version 8.2.0, use the following in the Podfile
        (but note that pod versions may not always match the C++ SDK version):

        ```
          pod 'Firebase/Firestore', '8.2.0'
          pod 'Firebase/Auth', '8.2.0'
        ```

    -   From the testapp directory, install the CocoaPods listed in the Podfile
        by running,

        ```
          pod install
        ```

    -   Open the generated Xcode workspace (which now has the CocoaPods),

        ```
          open testapp.xcworkspace
        ```

    -   For further details please refer to the
        [general instructions for setting up an iOS app with Firebase](https://firebase.google.com/docs/ios/setup).

-   Register your iOS app with Firebase.

    -   Create a new app on the
        [Firebase console](https://firebase.google.com/console/), and attach
        your iOS app to it.
        -   You can use "com.google.firebase.cpp.firestore.testapp" as the iOS
            Bundle ID while you're testing. You can omit App Store ID while
            testing.
    -   Add the GoogleService-Info.plist that you downloaded from Firebase
        console to the testapp root directory. This file identifies your iOS app
        to the Firebase backend.
    -   In the Firebase console for your app, select "Auth", then enable
        "Anonymous". This will allow the testapp to use anonymous sign-in to
        authenticate with Firebase Firestore, which requires a signed-in user by
        default (an anonymous user will suffice).

-   Download the Firebase C++ SDK linked from
    [https://firebase.google.com/docs/cpp/setup](https://firebase.google.com/docs/cpp/setup)
    and unzip it to a directory of your choice.

-   Add the following frameworks from the Firebase C++ SDK to the project:

    -   xcframeworks/firebase.xcframework/ios-arm64_armv7/firebase.framework
    -   xcframeworks/firebase_firestore.xcframework/ios-arm64_armv7/firebase_firestore.framework
    -   xcframeworks/firebase_auth.xcframework/ios-arm64_armv7/firebase_auth.framework
    -   You will need to either,
        1.  Check "Copy items if needed" when adding the frameworks, or
        2.  Add the framework path in "Framework Search Paths"
            -   e.g. If you downloaded the Firebase C++ SDK to
                `/Users/me/firebase_cpp_sdk`, then you would add the path
                `/Users/me/firebase_cpp_sdk/xcframeworks/**`.
            -   To add the path, in XCode, select your project in the project
                navigator, then select your target in the main window. Select
                the "Build Settings" tab, and click "All" to see all the build
                settings. Scroll down to "Search Paths", and add your path to
                "Framework Search Paths".

-   In XCode, build & run the sample on an iOS device or simulator.

-   The testapp has no interative interface. The output of the app can be viewed
    via the console or on the device's display. In Xcode, select "View --> Debug
    Area --> Activate Console" from the menu to view the console.

### Android

-   Register your Android app with Firebase.

    -   Create a new app on the
        [Firebase console](https://firebase.google.com/console/), and attach
        your Android app to it.

        -   You can use "com.google.firebase.cpp.firestore.testapp" as the
            Package Name while you're testing.
        -   To
            [generate a SHA1](https://developers.google.com/android/guides/client-auth)
            run this command on Mac and Linux,

            ```
            keytool -exportcert -list -v -alias androiddebugkey -keystore ~/.android/debug.keystore
            ```

            or this command on Windows,

            ```
            keytool -exportcert -list -v -alias androiddebugkey -keystore %USERPROFILE%\.android\debug.keystore
            ```

        -   If keytool reports that you do not have a debug.keystore, you can
            [create one with](http://developer.android.com/tools/publishing/app-signing.html#signing-manually),

            ```
            keytool -genkey -v -keystore ~/.android/debug.keystore -storepass android -alias androiddebugkey -keypass android -dname "CN=Android Debug,O=Android,C=US"
            ```

    -   Add the `google-services.json` file that you downloaded from Firebase
        console to the root directory of testapp. This file identifies your
        Android app to the Firebase backend.

    -   In the Firebase console for your app, select "Auth", then enable
        "Anonymous". This will allow the testapp to use anonymous sign-in to
        authenticate with Firebase Firestore, which requires a signed-in user by
        default (an anonymous user will suffice).

    -   For further details please refer to the
        [general instructions for setting up an Android app with Firebase](https://firebase.google.com/docs/android/setup).

-   Download the Firebase C++ SDK linked from
    [https://firebase.google.com/docs/cpp/setup](https://firebase.google.com/docs/cpp/setup)
    and unzip it to a directory of your choice.

-   Configure the location of the Firebase C++ SDK by setting the
    firebase\_cpp\_sdk.dir Gradle property to the SDK install directory. For
    example, in the project directory:

    ```
    echo "systemProp.firebase\_cpp\_sdk.dir=/User/$USER/firebase\_cpp\_sdk" >> gradle.properties
    ```

-   Ensure the Android SDK and NDK locations are set in Android Studio.

    -   From the Android Studio launch menu, go to `File/Project Structure...`
        or `Configure/Project Defaults/Project Structure...` (Shortcut:
        Control + Alt + Shift + S on windows, Command + ";" on a mac) and
        download the SDK and NDK if the locations are not yet set.

-   Open *build.gradle* in Android Studio.

    -   From the Android Studio launch menu, "Open an existing Android Studio
        project", and select `build.gradle`.

-   Install the SDK Platforms that Android Studio reports missing.

-   Build the testapp and run it on an Android device or emulator.

-   The testapp has no interactive interface. The output of the app can be
    viewed on the device's display, or in the logcat output of Android studio or
    by running "adb logcat *:W android_main firebase" from the command line.

### Desktop

-   Register your app with Firebase.
    -   Create a new app on the
        [Firebase console](https://firebase.google.com/console/), following the
        above instructions for Android or iOS.
    -   If you have an Android project, add the `google-services.json` file that
        you downloaded from the Firebase console to the root directory of the
        testapp.
    -   If you have an iOS project, and don't wish to use an Android project,
        you can use the Python script `generate_xml_from_google_services_json.py
        --plist`, located in the Firebase C++ SDK, to convert your
        `GoogleService-Info.plist` file into a `google-services-desktop.json`
        file, which can then be placed in the root directory of the testapp.
-   Download the Firebase C++ SDK linked from
    [https://firebase.google.com/docs/cpp/setup](https://firebase.google.com/docs/cpp/setup)
    and unzip it to a directory of your choice.
-   Configure the testapp with the location of the Firebase C++ SDK. This can be
    done a couple different ways (in highest to lowest priority):
    -   When invoking cmake, pass in the location with
        -DFIREBASE_CPP_SDK_DIR=/path/to/firebase_cpp_sdk.
    -   Set an environment variable for FIREBASE_CPP_SDK_DIR to the path to use.
    -   Edit the CMakeLists.txt file, changing the FIREBASE_CPP_SDK_DIR path to
        the appropriate location.
-   From the testapp directory, generate the build files by running,

    ```
      cmake .
    ```

    If you want to use XCode, you can use -G"Xcode" to generate the project.
    Similarly, to use Visual Studio, -G"Visual Studio 15 2017". For more
    information, see
    [CMake generators](https://cmake.org/cmake/help/latest/manual/cmake-generators.7.html).

-   Build the testapp, by either opening the generated project file based on the
    platform, or running,

    ```
      cmake --build .
    ```

-   Execute the testapp by running,

    ```
      ./desktop_testapp
    ```

    Note that the executable might be under another directory, such as Debug.

-   The testapp has no user interface, but the output can be viewed via the
    console.

Support
-------

[https://firebase.google.com/support/](https://firebase.google.com/support/)

License
-------

Copyright 2016-2019 Google LLC Licensed to the Apache Software Foundation (ASF)
under one or more contributor license agreements. See the NOTICE file
distributed with this work for additional information regarding copyright
ownership. The ASF licenses this file to you under the Apache License, Version
2.0 (the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
License for the specific language governing permissions and limitations under
the License.
